﻿using System;
using System.Collections.Generic;

using DevExpress.Map;
using DevExpress.XtraMap;

namespace TestProject
{
    /// <summary>
    /// 클러스터
    /// </summary>
    public class Cluster
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 지원 좌표 위치 리스트
        /// </summary>
        private List<ISupportCoordLocation> supportCoordLocationList;

        /// <summary>
        /// 중심점
        /// </summary>
        private MapPoint centerPoint;

        /// <summary>
        /// 가장 가까운 클러스터
        /// </summary>
        private Cluster closestCluster;

        /// <summary>
        /// 가장 가까운 거리
        /// </summary>
        private double distanceToClosest;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Property
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 지원 좌표 위치 리스트 - SupportCoordLocationList

        /// <summary>
        /// 지원 좌표 위치 리스트
        /// </summary>
        public List<ISupportCoordLocation> SupportCoordLocationList
        {
            get
            {
                return this.supportCoordLocationList;
            }
        }

        #endregion
        #region 중심점 - CenterPoint

        /// <summary>
        /// 중심점
        /// </summary>
        public MapPoint CenterPoint
        {
            get
            {
                return this.centerPoint;
            }
        }

        #endregion
        #region 가장 가까운 클러스터 - ClosestCluster

        /// <summary>
        /// 가장 가까운 클러스터
        /// </summary>
        public Cluster ClosestCluster
        {
            get
            {
                return this.closestCluster;
            }
            set
            {
                this.closestCluster = value;

                this.distanceToClosest = GetDistance(this.closestCluster);
            }
        }

        #endregion
        #region 가장 가까운 거리 - DistanceToClosest

        /// <summary>
        /// 가장 가까운 거리
        /// </summary>
        public double DistanceToClosest
        {
            get
            {
                return this.distanceToClosest;
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - Cluster(supportCoordLocationList)

        /// <summary>
        /// 생성자
        /// </summary>
        /// <param name="supportCoordLocationList">지원 좌표 위치 리스트</param>
        public Cluster(List<ISupportCoordLocation> supportCoordLocationList)
        {
            this.supportCoordLocationList = supportCoordLocationList;
            this.centerPoint              = CalculateCenterPoint(supportCoordLocationList);
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Static
        //////////////////////////////////////////////////////////////////////////////// Public

        #region 초기화하기 - Initialize(supportCoordLocation)

        /// <summary>
        /// 초기화하기
        /// </summary>
        /// <param name="supportCoordLocation">지원 좌표 위치</param>
        /// <returns>클러스터</returns>
        public static Cluster Initialize(ISupportCoordLocation supportCoordLocation)
        {
            List<ISupportCoordLocation> supportCoordLocationList = new List<ISupportCoordLocation>();

            supportCoordLocationList.Add(supportCoordLocation);

            return new Cluster(supportCoordLocationList);
        }

        #endregion
        #region 병합하기 - Cluster Merge(cluster1, cluster2)

        /// <summary>
        /// 병합하기
        /// </summary>
        /// <param name="cluster1">클러스터 1</param>
        /// <param name="cluster2">클러스터 2</param>
        /// <returns>클러스터</returns>
        public static Cluster Merge(Cluster cluster1, Cluster cluster2)
        {
            List<ISupportCoordLocation> list = new List<ISupportCoordLocation>(cluster1.SupportCoordLocationList);

            list.AddRange(cluster2.SupportCoordLocationList);

            return new Cluster(list);
        }

        #endregion
        #region 중심점 계산하기 - CalculateCenterPoint(supportCoordLocationList)

        /// <summary>
        /// 중심점 계산하기
        /// </summary>
        /// <param name="supportCoordLocationList">지원 좌표 위치 리스트</param>
        /// <returns>중심점</returns>
        public static MapPoint CalculateCenterPoint(List<ISupportCoordLocation> supportCoordLocationList)
        {
            double meanX = 0;
            double meanY = 0;

            foreach(ISupportCoordLocation supportCoordLocation in supportCoordLocationList)
            {
                meanX += supportCoordLocation.Location.GetX();
                meanY += supportCoordLocation.Location.GetY();
            }

            meanX /= supportCoordLocationList.Count;
            meanY /= supportCoordLocationList.Count;

            return new MapPoint(meanX, meanY);
        }

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Instance
        //////////////////////////////////////////////////////////////////////////////// Public

        #region 거리 구하기 - GetDistance(cluster)

        /// <summary>
        /// 거리 구하기
        /// </summary>
        /// <param name="cluster">클러스터</param>
        /// <returns>거리</returns>
        public double GetDistance(Cluster cluster)
        {
            return Math.Sqrt
            (
                (cluster.CenterPoint.X - CenterPoint.X) * (cluster.CenterPoint.X - CenterPoint.X) +
                (cluster.CenterPoint.Y - CenterPoint.Y) * (cluster.CenterPoint.Y - CenterPoint.Y)
            );
        }

        #endregion
    }
}